/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package brooklyn.demo;

import java.io.IOException;

import org.marre.SmsSender;
import org.marre.sms.SmsException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Sms {

    private static final Logger LOG = LoggerFactory.getLogger(Sms.class);

    private final String username;
    private final String password;
    private final String apiid;
    private final String sender;

    public Sms(String username, String password, String apiid) {
        this.username = username;
        this.password = password;
        this.apiid = apiid;
        this.sender = null;
    }
    
    /**
     * 
     * @param receiver International number to reciever without leading "+"
     * @param msg      The message that you want to send
     * @throws IOException 
     * @throws SmsException 
     */
    public void sendSms(String receiver, String msg) throws SmsException, IOException {
        // Send SMS with clickatell
        SmsSender smsSender = SmsSender.getClickatellSender(username, password, apiid);

        smsSender.connect();
        try {
            String msgid = smsSender.sendTextSms(msg, receiver, sender);
            LOG.debug("Sent SMS via {}@clickatell to {}, msg {}; id {}", new Object[] {username, receiver, msg, msgid});
        } finally {
            smsSender.disconnect();
        }
    }
}
